package edu.swarthmore.cs.cs71.lecture.junit;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class LightsOutModelImpl implements LightsOutModel {
    private boolean gameOver;
    private boolean[][] grid;
    private int moveCount;
    private List<LightsOutModelListener> listeners;

    public LightsOutModelImpl() {
        Random random = new Random();
        this.listeners = new ArrayList<>();
        this.grid = new boolean[LightsOutModel.BOARD_SIZE][LightsOutModel.BOARD_SIZE];
        this.gameOver = true;
        for (int i=0;i<this.grid.length;i++) {
            for (int j=0;j<this.grid[i].length;j++) {
                this.grid[i][j] = random.nextBoolean();
                this.gameOver = this.gameOver && this.grid[i][j];
            }
        }
        this.moveCount = 0;
    }

    private void notifyListeners(int x,int y) {
        for (LightsOutModelListener listener : this.listeners) {
            listener.squareChanged(x,y);
        }
    }

    @Override
    public void toggleSpace(int x, int y) {
        this.grid[x][y] = !this.grid[x][y];
        notifyListeners(x,y);
        if (x>0) {
            this.grid[x-1][y] = !this.grid[x-1][y];
            notifyListeners(x-1,y);
        }
        if (x<LightsOutModel.BOARD_SIZE-1) {
            this.grid[x+1][y] = !this.grid[x+1][y];
            notifyListeners(x+1,y);
        }
        if (y>0) {
            this.grid[x][y-1] = !this.grid[x][y-1];
            notifyListeners(x,y-1);
        }
        if (y<LightsOutModel.BOARD_SIZE-1) {
            this.grid[x][y+1] = !this.grid[x][y+1];
            notifyListeners(x,y+1);
        }
        this.moveCount++;
        boolean acc = true;
        for (int ix=0;ix<LightsOutModel.BOARD_SIZE;ix++) {
            for (int iy=0;iy<LightsOutModel.BOARD_SIZE;iy++) {
                acc &= this.grid[ix][iy];
            }
        }
        this.gameOver |= acc;
    }

    @Override
    public int getMoveCount() {
        return this.moveCount;
    }

    @Override
    public boolean getSpaceState(int x, int y) {
        return this.grid[x][y];
    }

    @Override
    public void addListener(LightsOutModelListener listener) {
        this.listeners.add(listener);
    }
}
